<?php

/**
 * @file
 * Provides common batch functions for UploadExtensions check.
 */

use Drupal\field\Entity\FieldConfig;
use Drupal\security_review\CheckResult;

/**
 * Batch operation for UploadExtensions, 10 at a time.
 *
 * @param array $field_ids
 *   Array of field_config IDs.
 * @param string $check
 *   Name of the check being run, defaults to View Access.
 * @param array $hush_upload_extensions
 *   Array of config set upload_extensions to ignore.
 * @param array $context
 *   Batch context that stores info.
 */
function security_review_process_upload_extensions(array $field_ids, string $check, array $hush_upload_extensions, array &$context): void {
  if (empty($context['sandbox'])) {
    $context['sandbox'] = [];
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($field_ids);
    $context['results']['findings'] = [];
    $context['results']['hushed'] = [];
  }

  $security_data = \Drupal::service('security_review.data');
  $ids = array_slice($field_ids, $context['sandbox']['progress'], 10);

  // Check field configuration entities.
  foreach (FieldConfig::loadMultiple($ids) as $field_config) {
    $extensions = $field_config->getSetting('file_extensions');
    if ($extensions != NULL) {
      $extensions = explode(' ', $extensions);
      $intersect = array_intersect($extensions, $security_data->unsafeExtensions());
      // $intersect holds the unsafe extensions this entity allows.
      foreach ($intersect as $unsafe_extension) {
        if (array_key_exists($unsafe_extension, $hush_upload_extensions)) {
          $hushed_findings[$field_config->id()][] = $unsafe_extension;
          $hushed_findings[$field_config->id()]['reason'] = $hush_upload_extensions[$unsafe_extension];
          continue;
        }
        $findings[$field_config->id()][] = $unsafe_extension;
      }
    }

    if (!empty($findings)) {
      $context['results']['findings'] = array_merge($context['results']['findings'], $findings);
    }

    if (!empty($hushed_findings)) {
      $context['results']['hushed'] = array_merge($context['results']['hushed'], $hushed_findings);
    }

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['message'] = t('Running Batch "@check" @field_id',
      ['@check' => $check, '@field_id' => $field_config->id()]
    );
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Callback for finishing a bulk check.
 */
function security_review_process_upload_extensions_finishied($success, $results, $operations): void {
  $plugin = \Drupal::service('plugin.manager.security_review.security_check')->createInstance('upload_extensions');
  if ($success) {
    $result = CheckResult::SUCCESS;
    if (!empty($results['findings'])) {
      $result = CheckResult::FAIL;
    }
    $plugin->createResult($result, $results['findings'], NULL, $results['hushed']);
  }
}
