<?php

/**
 * @file
 * Provides common batch functions for ViewsAccess check.
 */

use Drupal\security_review\CheckResult;
use Drupal\views\Entity\View;

/**
 * Batch operation for ViewsAccess, 5 at a time.
 *
 * @param array $vids
 *   Array of view IDs.
 * @param string $check
 *   Name of the check being run, defaults to View Access.
 * @param bool $ignore_defaults
 *   Config setting if defaults should be ignored.
 * @param array $context
 *   Batch context that stores info.
 */
function security_review_process_views(array $vids, string $check, bool $ignore_defaults, array &$context): void {
  if (empty($context['sandbox'])) {
    $context['sandbox'] = [];
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($vids);
    $context['results']['findings'] = [];
  }

  $ids = array_slice($vids, $context['sandbox']['progress'], 5);
  // Retrieve the next group.
  $views = View::loadMultiple($ids);
  $default = NULL;
  foreach ($views as $view) {
    if ($view->status()) {
      $findings = [];
      foreach ($view->get('display') as $display_name => $display) {
        $access = $display['display_options']['access'] ?? $default;
        if ($display_name == 'default' && $ignore_defaults) {
          $default = $access;
        }
        elseif (isset($access) && $access['type'] == 'none') {
          // Access is not controlled for this display.
          $findings[$view->id()][] = $display_name;
        }
      }

      if (!empty($findings)) {
        $context['results']['findings'] = array_merge($context['results']['findings'], $findings);
      }
    }

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['message'] = t('Running Batch "@check" @view_id',
      ['@check' => $check, '@view_id' => $view->id()]
    );
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Callback for finishing a bulk check.
 */
function security_review_process_views_finished($success, $results, $operations): void {
  $plugin = \Drupal::service('plugin.manager.security_review.security_check')->createInstance('views_access');
  if ($success) {
    $result = CheckResult::SUCCESS;
    if (!empty($results['findings'])) {
      $result = CheckResult::FAIL;
    }
    $plugin->createResult($result, $results['findings']);
  }
}
