<?php

declare(strict_types = 1);

namespace Drupal\security_review\Plugin\SecurityCheck;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Utility\Html;
use Drupal\security_review\SecurityCheckBase;

/**
 * Checks if a user has the same name and password.
 *
 * @SecurityCheck(
 *   id = "name_passwords",
 *   title = @Translation("Name password check"),
 *   description = @Translation("Checks if a user has the same name and password."),
 *   namespace = @Translation("Security Review"),
 *   success_message = @Translation("No users, with matching username and password, found."),
 *   failure_message = @Translation("Users, with matching username and password, found."),
 *   help = {
 *     @Translation("Verifies that users have not set their password to be the same as their username."),
 *   }
 * )
 */
class NamePasswords extends SecurityCheckBase {

  /**
   * {@inheritdoc}
   */
  public function run(bool $cli = FALSE): void {
    try {
      $user_ids = $this->entityTypeManager->getStorage('user')
        ->getQuery()
        ->accessCheck()
        ->condition('uid', 0, '<>')
        ->execute();

      $batch = [
        'operations' => [
          [
            'security_review_process_users',
            [$user_ids, "Name Passwords"],
          ],
        ],
        'file' => $this->extensionPathResolver->getPath('module', 'security_review') . '/include_batches/name_passwords.batch.inc',
        'finished' => 'security_review_process_users_finished',
        'title' => $this->t('Processing Name Passwords Check'),
        'init_message' => $this->t('Name Passwords Check batch is starting.'),
        'progress_message' => $this->t('Processed @current out of @total.'),
        'error_message' => $this->t('Name Passwords Check batch has encountered an error.'),
      ];

      batch_set($batch);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException) {
      $this->messenger()->addError('Error running name_passwords check.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDetails(array $findings = [], array $hushed = [], bool $returnString = FALSE): array|string {
    if (empty($findings)) {
      return [];
    }

    $output = $returnString ? '' : [];
    $paragraphs = [];
    $paragraphs[] = $this->t('The following user(s) has their password set to be the same as their username');
    $user_list = [];
    foreach ($findings as $user) {
      $user_list[] = Html::escape($user);
    }

    $paragraphs[] = $this->t('Consider installing the <a href="https://www.drupal.org/project/password_policy">Password Policy</a> module, to enforce users to have a stronger password.');

    if ($returnString) {
      $output .= implode("", $paragraphs) . implode("", $user_list);
    }
    else {
      $output[] = [
        '#theme' => 'check_evaluation',
        '#paragraphs' => $paragraphs,
        '#items' => $user_list,
      ];
    }

    return $output;
  }

}
