<?php

declare(strict_types = 1);

namespace Drupal\security_review\Plugin\SecurityCheck;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\field\Entity\FieldConfig;
use Drupal\security_review\CheckResult;
use Drupal\security_review\SecurityCheckBase;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

/**
 * Checks for unsafe extensions in the allowed extensions settings of fields.
 *
 * @SecurityCheck(
 *   id = "upload_extensions",
 *   title = @Translation("Allowed upload extensions"),
 *   description = @Translation("Checks for unsafe extensions in the allowed extensions settings of fields."),
 *   namespace = @Translation("Security Review"),
 *   success_message = @Translation("Only safe extensions are allowed for uploaded files and images."),
 *   failure_message = @Translation("Unsafe file extensions are allowed in uploads."),
 *   info_message = @Translation("Module field is not enabled."),
 *   help = {
 *     @Translation("File and image fields allow for uploaded files. Some extensions are considered dangerous because the files can be evaluated and then executed in the browser. A malicious user could use this opening to gain control of your site. Review <a href=""/admin/reports/fields"">all fields on your site</a>."),
 *   }
 * )
 */
class UploadExtensions extends SecurityCheckBase {

  /**
   * {@inheritdoc}
   */
  public function run(bool $cli = FALSE): void {
    if (!$this->moduleHandler->moduleExists('field')) {
      $this->createResult(CheckResult::INFO);
    }
    else {
      try {
        $config = $this->securityReview->getCheckSettings($this->pluginId);
        $hush_upload_extensions = $config['hush_upload_extensions'] ?? [];

        $field_ids = $this->entityTypeManager->getStorage('field_config')
          ->getQuery()
          ->accessCheck()
          ->execute();
        $batch = [
          'operations' => [
            [
              'security_review_process_upload_extensions',
              [
                $field_ids,
                "Upload Extensions",
                $this->getHushedExtensions($hush_upload_extensions),
              ],
            ],
          ],
          'file' => $this->extensionPathResolver->getPath('module', 'security_review') . '/include_batches/upload_extension.batch.inc',
          'finished' => 'security_review_process_upload_extensions_finishied',
          'title' => $this->t('Processing Upload Extensions'),
          'init_message' => $this->t('Upload extension batch is starting.'),
          'progress_message' => $this->t('Processed @current out of @total.'),
          'error_message' => $this->t('Upload extensions batch has encountered an error.'),
        ];

        batch_set($batch);
      }
      catch (InvalidPluginDefinitionException | PluginNotFoundException) {
        $this->messenger()->addError('Error getting field_config in upload_extensions check');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $config = $this->securityReview->getCheckSettings($this->pluginId);
    $known_risky_upload_extensions = $config['hush_upload_extensions'] ?? [];
    $form = [];
    $form['hush_upload_extensions'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Hush known risky extensions'),
      '#description' => $this->t('File upload extensions to be skipped in future runs. Enter one value per line, in the format extension|reason.'),
      '#default_value' => implode("\n", $known_risky_upload_extensions),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array $values): void {
    $hushed['hush_upload_extensions'] = [];
    if (!empty($values['hush_upload_extensions'])) {
      $hushed['hush_upload_extensions'] = preg_split("/\r\n|\n|\r/", $values['hush_upload_extensions']);
    }
    $this->securityReview->setCheckSettings($this->pluginId, $hushed);
  }

  /**
   * {@inheritdoc}
   */
  public function getDetails(array $findings, array $hushed = [], bool $returnString = FALSE): array|string {
    if (empty($findings) && empty($hushed)) {
      return [];
    }

    $output = $returnString ? '' : [];
    $paragraphs = [];

    $items = [];
    foreach ($findings as $entity_id => $unsafe_extensions) {
      $entity = FieldConfig::load($entity_id);
      foreach ($unsafe_extensions as $extension) {
        $item = $this->t(
          'Review extension <strong>@extension</strong> in <strong>@field</strong> field on bundle <strong>@bundle</strong> of type <strong>@type</strong>',
          [
            '@extension' => $extension,
            '@field' => $entity->label(),
            '@bundle' => $entity->getTargetBundle(),
            '@type' => $entity->getTargetEntityTypeId(),
          ]
        );

        // Try to get an edit url.
        try {
          $url_params = ['field_config' => $entity->id()];
          if (in_array($entity->getTargetEntityTypeId(), ['node', 'media'])) {
            $url_params[$entity->getTargetEntityTypeId() . '_type'] = $entity->getTargetBundle();
          }
          $items[] = Link::createFromRoute(
            $item,
            sprintf('entity.field_config.%s_field_edit_form', $entity->getTargetEntityTypeId()),
            $url_params
          );
        }
        catch (RouteNotFoundException) {
          $items[] = $item;
        }
      }
    }

    $hushed_items = [];
    foreach ($hushed as $entity_id => $hushed_item) {
      $entity = FieldConfig::load($entity_id);
      $item = $this->t(
        'Extension <strong>@extension</strong> in <strong>@field</strong> field on bundle <strong>@bundle</strong> of type <strong>@type</strong>. Reason: @reason',
        [
          '@extension' => $hushed_item[0],
          '@field' => $entity->label(),
          '@bundle' => $entity->getTargetBundle(),
          '@type' => $entity->getTargetEntityTypeId(),
          '@reason' => $hushed_item['reason'],
        ]
      );

      // Try to get an edit url.
      try {
        $url_params = ['field_config' => $entity->id()];
        if (in_array($entity->getTargetEntityTypeId(), ['node', 'media'])) {
          $url_params[$entity->getTargetEntityTypeId() . '_type'] = $entity->getTargetBundle();
        }
        $hushed_items[] = Link::createFromRoute(
          $item,
          sprintf('entity.field_config.%s_field_edit_form', $entity->getTargetEntityTypeId()),
          $url_params
        );
      }
      catch (RouteNotFoundException) {
        $hushed_items[] = $item;
      }
    }

    if ($returnString) {
      $output .= implode("", $paragraphs) . implode("", $items);
    }
    else {
      $output[] = [
        '#theme' => 'check_evaluation',
        '#paragraphs' => $paragraphs,
        '#items' => $items,
        '#hushed_items' => $hushed_items,
      ];
    }

    return $output;
  }

  /**
   * Generates an array of hushed extensions.
   *
   * @param array $values
   *   Array of values from config where key is numerical. Turn this into
   *   something more useable.
   *
   * @return array
   *   A key|value array of the hushed values.
   */
  protected function getHushedExtensions(array $values): array {
    $lines = [];
    foreach ($values as $value) {
      $parts = explode('|', $value);
      $lines[$parts[0]] = $parts[1];
    }
    return $lines;
  }

}
