<?php

declare(strict_types = 1);

namespace Drupal\security_review\Plugin\SecurityCheck;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\security_review\CheckResult;
use Drupal\security_review\SecurityCheckBase;
use Drupal\views\Entity\View;

/**
 * Checks for Views that do not check access.
 *
 * @SecurityCheck(
 *   id = "views_access",
 *   title = @Translation("Views Access"),
 *   description = @Translation("Checks for Views that do not check access."),
 *   namespace = @Translation("Security Review"),
 *   success_message = @Translation("Views are access controlled."),
 *   failure_message = @Translation("There are Views that do not provide any access checks."),
 *   info_message = @Translation("Module views is not enabled."),
 *   help = {
 *     @Translation("Views can check if the user is allowed access to the content. It is recommended that all Views implement some amount of access control, at a minimum checking for the permission 'access content'."),
 *   }
 * )
 */
class ViewsAccess extends SecurityCheckBase {

  /**
   * {@inheritdoc}
   */
  public function run(bool $cli = FALSE): void {
    // If views is not enabled return with INFO.
    if (!$this->moduleHandler->moduleExists('views')) {
      $this->createResult(CheckResult::INFO);
    }
    else {
      try {
        $config = $this->securityReview->getCheckSettings($this->pluginId);
        $ignore_default = $config['ignore_default'] ?? FALSE;

        $vids = $this->entityTypeManager->getStorage('view')
          ->getQuery()
          ->accessCheck()
          ->execute();
        $batch = [
          'operations' => [
            [
              'security_review_process_views',
              [$vids, "Views Access", $ignore_default],
            ],
          ],
          'file' => $this->extensionPathResolver->getPath('module', 'security_review') . '/include_batches/views_access.batch.inc',
          'finished' => 'security_review_process_views_finished',
          'title' => $this->t('Processing Views Access'),
          'init_message' => $this->t('Views Access batch is starting.'),
          'progress_message' => $this->t('Processed @current out of @total.'),
          'error_message' => $this->t('Views Access batch has encountered an error.'),
        ];

        batch_set($batch);
      }
      catch (InvalidPluginDefinitionException | PluginNotFoundException) {
        $this->messenger()->addError('Error getting views in view_access check');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $config = $this->securityReview->getCheckSettings($this->pluginId);
    $ignore_default = $config['ignore_default'] ?? FALSE;
    $form = [];
    $form['ignore_default'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ignore default view'),
      '#description' => $this->t('Check to ignore default views.'),
      '#default_value' => $ignore_default,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array $values): void {
    if (isset($values['ignore_default'])) {
      $values['ignore_default'] = (bool) $values['ignore_default'];
    }
    $this->securityReview->setCheckSettings($this->pluginId, $values);
  }

  /**
   * {@inheritdoc}
   */
  public function getDetails(array $findings, array $hushed = [], bool $returnString = FALSE): array|string {
    if (empty($findings)) {
      return [];
    }

    $output = $returnString ? '' : [];
    $views_ui_enabled = $this->moduleHandler->moduleExists('views_ui');
    $paragraphs = [];
    $paragraphs[] = $this->t('The following View displays do not check access.');
    $items = [];
    foreach ($findings as $view_id => $displays) {
      $view = View::load($view_id);
      /** @var \Drupal\views\Entity\View $view */

      foreach ($displays as $display) {
        $label = $view->label() . ': ' . $display;
        $items[] = $views_ui_enabled ?
          Link::createFromRoute(
            $label,
            'entity.view.edit_display_form',
            [
              'view' => $view_id,
              'display_id' => $display,
            ]
          ) :
          $label;
      }
    }

    if ($returnString) {
      $output .= implode("", $paragraphs) . implode("", $items);
    }
    else {
      $output[] = [
        '#theme' => 'check_evaluation',
        '#paragraphs' => $paragraphs,
        '#items' => $items,
      ];
    }

    return $output;
  }

}
